/*
 * @(#) SimpleJmsXa.java
 *
 * JOTM: Java Open Transaction Manager 
 *
 *
 * This module was orginally developed by 
 *
 *  - INRIA (www.inria.fr)inside the ObjectWeb Consortium 
 *    (http://www.objectweb.org)
 * 
 * --------------------------------------------------------------------------
 *  The original code and portions created by INRIA are 
 *  Copyright (c) 2002 INRIA  
 *  All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * -Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. 
 *
 * -Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 *-----------------------------------------------------------------------------
 * $Id: SimpleJmsXa.java,v 1.3 2003/12/05 20:07:06 trentshue Exp $
 *-----------------------------------------------------------------------------
 */


import javax.jms.Queue;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.TransactionManager;

import org.objectweb.jonas_jms.api.JmsManager;
import org.objectweb.jonas_jms.JmsManagerImpl;
import org.objectweb.jonas_jms.TraceJms;

import java.io.PrintWriter;

/**
 * Created on Mar 4, 2002
 * @author  Christophe Ney - cney@batisseurs.com
 */
public class SimpleJmsXa {

    public static void main(String[] args) {

        // parse args
        String jmsAdminClassName = "org.objectweb.jonas_jms.JmsAdminForJoram";
        if (args.length>0) {
            jmsAdminClassName = args[0];
        }

        // get initial context
        Context ctx = null;
        try {
            ctx = new InitialContext();
        } catch (NamingException e) {
            System.err.println("No Initial Context");
            e.printStackTrace();
            System.exit(1);
        }

        System.out.println("[SimpleJmsXa] lookup the TransactionManager.");
        // get transaction manager
        TransactionManager tm = null;
        try {
            tm = (TransactionManager) ctx.lookup("TransactionManager");
        } catch (NamingException e) {
            System.err.println("TransactionManager not found");
            e.printStackTrace();
            System.exit(1);
        }

        System.out.println("[SimpleJmsXa] start the JMS server.");
        // load the JMS Resource Adaptor using the JOnAS JmsAdministration SPI
        // The SPI is made of one interface org.objectweb.jonas_jms.api.JmsAdministration
        Class cAdmin = null;
        try {
            cAdmin = Class.forName(jmsAdminClassName);
        } catch (ClassNotFoundException e) {
            System.err.println("Cannot load class "+jmsAdminClassName);
            e.printStackTrace();
            System.exit(1);
        }

        // the MOM is collocated in the VM (could be remote)
        boolean collocated = true;
        String url = null;

        // init the JMS Resource Manager using the JOnAS JmsManager API
        // The API is made of only one interface org.objectweb.jonas_jms.api.JmsManager
        TraceJms.setVerbose(false);
        TraceJms.setDebug(false);
        TraceJms.setLogWriter(new PrintWriter(System.out,true));
        JmsManager jmsMg = JmsManagerImpl.getJmsManager();
        try {
            jmsMg.init(cAdmin, collocated, url, tm);
        } catch (Exception e) {
            System.err.println("Cannot initialize JmsManager");
            e.printStackTrace();
            System.exit(1);
        }
        System.out.println("[SimpleJmsXa] JMS server started.");                

        System.out.println("[SimpleJmsXa] create JMS objects, register them in JOTM and bind them.");
        try {
            // Rebind theQuueConnectionFactory for JNDI access
            // It is similar to binding a datasource for a given driver
            ctx.rebind("theQueueConnectionFactory", jmsMg.getQueueConnectionFactory());

            // Create and rebind theQueue
            // It is similar to creating a TABLE in a database, but the name
            // space is used to store the name
            Queue queue = jmsMg.createQueue("theQueue");
            ctx.rebind("theQueue", queue);

        } catch (NamingException e) {
            System.err.println("Cannot rebind JMS objects");
            e.printStackTrace();
            System.exit(1);
        } catch (Exception e) {
            System.err.println("Cannot create Queue");
            e.printStackTrace();
            System.exit(1);
        }
        System.out.println("[SimpleJmsXa] JMS objects available.");

        System.out.println("[SimpleJmsXa] start simple sender.");
        // send a few messages on theQueue (current thread for now)
        new SimpleSender().run();

        System.out.println("[SimpleJmsXa] start simple receiver.");
        // receive messages found in theQueue (SimpleReceiver is a MessageListener)
        new SimpleReceiver().run();

        // stop the JMS Manager
        try {
            ctx.unbind("theQueueConnectionFactory");
            ctx.unbind("theQueue");
            jmsMg.stop();
        } catch (Exception e) {
            System.err.println("Error while shutting down");
            e.printStackTrace();
            System.exit(1);
        }

        System.out.println("[SimpleJmsXa] JMS server stopped");
        // stop the VM since ReferenceWarpper used for binding Reference in JNDI
        // are not unexported : jdk bug
        System.exit(0);
    }
}
